#line 82 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"
#include <stdint.h>
#include "graymap.h"

#line 6 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"

static uint8_t gm_red_shift, gm_blue_shift, gm_green_shift;
static uint32_t gm_fg, gm_bg;
static uint32_t colors[256];

#line 49 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"

static void gmap_set_colors(uint32_t fg, uint32_t bg)
{
  uint32_t bR,bG,bB;
  uint32_t fR,fG,fB;
  uint32_t pR,pG,pB;
  double V;
  int i;

  if (gm_fg==fg&&gm_bg==bg) return ;
  gm_fg= fg;
  gm_bg= bg;

  bR= (bg>>16)&0xff; bG= (bg>>8)&0xff; bB= (bg)&0xff;
  fR= (fg>>16)&0xff; fG= (fg>>8)&0xff; fB= (fg)&0xff;
  
  for(i=0;i<256;i++)
  {
    V= i / 255.0;
    pR= (int) (fR*V+bR*(1-V)) & 0xff;
    pG= (int) (fG*V+bG*(1-V)) & 0xff;
    pB= (int) (fB*V+bB*(1-V)) & 0xff;

    colors[i]=  (pR<<gm_red_shift) | (pG<<gm_green_shift) |
                (pB<<gm_blue_shift);
  }
}

#line 12 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"

void graymap_init(int red_shift,int green_shift, int blue_shift)
{
  gm_red_shift= red_shift;
  gm_blue_shift= blue_shift;
  gm_green_shift= green_shift;
}

void graymap_draw(uint32_t *dst, uint32_t dstW, uint32_t dstH,
               int dstX, int dstY,
               const uint8_t  *src, uint32_t srcW, uint32_t srcH,
               int srcX, int srcY,
               uint32_t fg, uint32_t bg,
               int W, int H)
{
  uint32_t r,c;

  if (dstX<0) { W+=dstX; dstX= 0; }
  if (srcX<0) { W+=srcX; srcX= 0; }
  if (dstY<0) { H+=dstY; dstY= 0; }
  if (srcY<0) { H+=srcY; srcY= 0; }

  if (dstX+W>dstW) W= dstW-dstX; 
  if (dstY+H>dstH) H= dstH-dstY; 

  if (srcX+W>srcW) W= srcW-srcX;
  if (srcY+H>srcH) H= srcH-srcY;

  if (W<=0 || H<=0) return ;
  gmap_set_colors(fg, bg);

  for(r=0;r<H;r++)
    for(c=0;c<W;c++)
      dst[(r+dstY)*dstW + (c+dstX)]= colors[ src[(r+srcY)*srcW+(c+srcX)] ];
}

#line 93 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"

void blend_alpha_image(uint32_t *dst,  int dstW, int dstH,
                                       int dstX, int dstY,
                       uint32_t *src,  int srcW, int srcH,
                                       int srcX, int srcY,
                       int W, int H, uint32_t bg)
{
  uint32_t bR,bG,bB;
  uint32_t fR,fG,fB,fA,fP;
  uint32_t sr,sg,sb;
  int r,c;
  double V;

  if (dstW<=0 || dstH<=0 || srcW<=0 || srcH<=0) return ;

  bR= (bg>>16)&0xff;
  bG= (bg>>8) &0xff;
  bB= bg      &0xff;

  if (dstX<0) { W+= dstX; dstX= 0; }
  if (srcX<0) { W+= srcX; srcX= 0; }
  if (dstY<0) { H+= dstY; dstY= 0; }
  if (srcY<0) { H+= srcY; srcY= 0; }

  if (W<=0 || H<=0 || dstX>=dstW || 
      dstY>=dstH || srcX>=srcW || srcY>=srcH) return ;

  if (dstX+W>dstW) W= dstW-dstX;
  if (srcX+W>srcW) W= srcW-srcX;
  if (dstY+H>dstH) H= dstH-dstY;
  if (srcY+H>srcH) H= srcH-srcY;

  sr= gm_red_shift;
  sg= gm_green_shift;
  sb= gm_blue_shift;

  for(r=0;r<H;r++)
    for(c=0;c<W;c++)
  {
    fP= src[ (r+srcY)*srcW + (c+srcX) ];
    fA= (fP>>24)&0xff;
    fR= (fP>>16)&0xff;
    fG= (fP>>8) &0xff;
    fB= (fP)    &0xff;
    V= fA/ 255.0;

    fR= (uint32_t) (fR*V+bR*(1-V)) &0xff;
    fG= (uint32_t) (fG*V+bG*(1-V)) &0xff;
    fB= (uint32_t) (fB*V+bB*(1-V)) &0xff;

    fP= (fR<<sr) | (fG<<sg) | (fB<<sb);

    dst[ (r+dstY)*dstW+ (c+dstX) ]= fP;
  }
}

void copy_area(uint32_t *dst,  int dstW, int dstH,
                               int dstX, int dstY,
               uint32_t *src,  int srcW, int srcH,
                               int srcX, int srcY,
                       int W, int H)
{
  int r,c;

  if (dstW<=0 || dstH<=0 || srcW<=0 || srcH<=0) return ;

  if (dstX<0) { W+= dstX; dstX= 0; }
  if (srcX<0) { W+= srcX; srcX= 0; }
  if (dstY<0) { H+= dstY; dstY= 0; }
  if (srcY<0) { H+= srcY; srcY= 0; }

  if (W<=0 || H<=0 || dstX>=dstW || 
      dstY>=dstH || srcX>=srcW || srcY>=srcH) return ;

  if (dstX+W>dstW) W= dstW-dstX;
  if (srcX+W>srcW) W= srcW-srcX;
  if (dstY+H>dstH) H= dstH-dstY;
  if (srcY+H>srcH) H= srcH-srcY;

  for(r=0;r<H;r++)
    for(c=0;c<W;c++)
      dst[ (r+dstY)*dstW+ (c+dstX) ]= src[ (r+srcY)*srcW + (c+srcX) ];
}

#line 178 "/home/dodo/tmp/mupanelV1.2-orig/src/graymap.u"

uint32_t server_color(uint32_t rgb)
{
  uint32_t R, G,B;
  R= (rgb>>16)&0xff;
  G= (rgb>>8) &0xff;
  B= rgb      &0xff;
  return (R<<gm_red_shift) | (G<<gm_green_shift) | (B<<gm_blue_shift);
}

void fill_rect(uint32_t *dst, uint32_t dstW, uint32_t dstH,
                              int x, int y, int w, int h,uint32_t color)
{
   uint32_t P,r,c;

   if (x<0) { w+= x; x= 0; }
   if (y<0) { h+= y; y= 0; }
   if (w<=0 || h<=0 || x>=dstW || y>=dstH) return ;
   if (x+w>dstW) w= dstW-x;
   if (y+h>dstH) h= dstH-y;
   P= server_color(color);
   for(r=0;r<h;r++)
     for(c=0;c<w;c++)
       dst[(r+y)*dstW+(c+x)]= P;
}

void xor_rect(uint32_t *dst, uint32_t dstW, uint32_t dstH,
                              int x, int y, int w, int h,uint32_t color)
{
   uint32_t P,r,c;

   if (x<0) { w+= x; x= 0; }
   if (y<0) { h+= y; y= 0; }
   if (w<=0 || h<=0 || x>=dstW || y>=dstH) return ;
   if (x+w>dstW) w= dstW-x;
   if (y+h>dstH) h= dstH-y;
   P= server_color(color);

   for(r=0;r<h;r++)
     for(c=0;c<w;c++)
       dst[(r+y)*dstW+(c+x)]^= P;
}

void draw_rect(uint32_t *dst, uint32_t dstW, uint32_t dstH,
                              int x, int y, int w, int h,uint32_t color)
{
   draw_vline(dst, dstW, dstH, x, y, y+h-1, color);
   draw_vline(dst, dstW, dstH, x+w-1, y, y+h-1, color);
   draw_hline(dst, dstW, dstH, x, x+w-1, y, color);
   draw_hline(dst, dstW, dstH, x, x+w-1, y+h-1, color);
}

void draw_vline(uint32_t *dst, uint32_t dstW, uint32_t dstH,
                              int x, int y1, int y2, uint32_t color)
{
  int t;
  uint32_t P;
  P= server_color(color);
  if (x<0 || x>=dstW) return ;
  if (y1>y2) { t= y2; y2= y1; y1= t; }
  if (y1<0) y1= 0;
  if (y2>=dstH) y2= dstH-1;
  while(y1<=y2) 
  {
     dst[ y1*dstW + x ]= P;
     y1++;
  }
}

void draw_hline(uint32_t *dst, uint32_t dstW, uint32_t dstH,
                              int x1, int x2, int y, uint32_t color)
{
  int t;
  uint32_t P;
  P= server_color(color);
  if (y<0 || y>=dstH) return ;
  if (x1>x2) { t= x2; x2= x1; x1= t; }
  if (x1<0) x1= 0;
  if (x2>=dstW) x2= dstW-1;
  while(x1<=x2) 
  {
     dst[ y*dstW + x1 ]= P;
     x1++;
  }
}

