#line 312 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"
#include <sys/ipc.h>
#include <syslog.h>
#include <sys/shm.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/XShm.h>
#include <stdio.h>
#include "x-basics.h"
#include "x-events.h"
#include "x-props.h"
#include "config.h"
#include "menu.h"
#include "txtren.h"
#include "graymap.h"

#line 21 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

typedef struct
{
  int type;  // 0- item 1- submenu
  menu_t *sub;
  int action;

  int width,height;
  uint32_t *image;

  int y;
} menu_item_t;

struct xmenu
{
  int n_items,s_items;
  menu_item_t *items;

  int width, height;
  uint32_t *image; 

  Window xwindow;
  XImage *ximage;
  int active_item;

  int x,y;
  menu_t *parent;
  void *root_click_handler;
  void *root_release_handler;

  int selected; // action number of the selected menu entry.
                // -1 if the menu was cancelled
                // This is present only in the top menu
  int released;
};

#line 129 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static int MO_left= 2, // left padding before the icon
           MO_right= 4, // right padding after the sub-menu indicator
           MO_mid = 2, // padding between icon and text
           MO_mid2 = 5, // padding between text and sub-menu indicator
           MO_vspc= 5,  // vertical spacing between items
           MO_icon_w= 16, // size of the icon area
           MO_icon_h= 16, 
           MO_font_size= 10, 
           MO_bg= 0xffffff,
           MO_fg= 0,
           MO_seph= 5, MO_sepv= 5, MO_sepcolor= 0xe0e0e0,
           MO_hilight= 0xffffff;
static char *MO_font_name= ":default";

#line 74 "menu.c"
static const unsigned int bi_menu_sub_width=6, bi_menu_sub_height= 10;
static const unsigned char bi_menu_sub[]= {
0x61,0x3c,0x00,0x00,0x00,0x00,0x4b,0x80,0x48,0x00,0x00,0x00,
0x00,0x41,0x80,0x51,0x02,0x00,0x00,0x00,0x36,0x7f,0x59,0x04,
0x00,0x00,0x00,0x2c,0x7d,0x56,0x00,0x00,0x00,0x2c,0x7d,0x5c,
0x00,0x00,0x36,0x7f,0x60,0x07,0x00,0x41,0x80,0x56,0x03,0x00,
0x48,0x80,0x4b,0x01,0x00,0x00,0x60,0x3e,0x00,0x00,0x00,0x00,
};
#line 355 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static u_font_t *menu_font;

#line 86 "menu.c"

static menu_item_t *alloc_item(menu_t *M);
static void render_item
  (menu_item_t *it, char *text,
   int icon_width, int icon_height, uint32_t *icon_data);
static void draw_item_icon(int W,int H, uint32_t *D,
                           int IW, int IH, uint32_t *ID);
static void draw_item_text
      (int W,int H, uint32_t *D, int tW, int tH, uint8_t *tD);
static void create_menu_window(menu_t *M, int x,int y, int istop);
static void menu_expose_event(menu_t *M, XExposeEvent *ev_expo);
static void menu_motion_event(menu_t *M, XMotionEvent *ev_moti);
static void menu_hilight_item(menu_t *M, int x,int y);
static void menu_hilight_bg(menu_t *M,menu_item_t *it,int events);
static void display_sub_menu(menu_t *M, int x,int y);
static void undisplay_sub_menu(menu_t *M);
static void menu_button_press(menu_t *root_menu, XButtonEvent *event);
#line 103 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static menu_item_t *alloc_item(menu_t *M)
{
  menu_item_t *it;
  if (M->n_items==M->s_items)
  {
    M->s_items+= M->s_items/2;
    M->items=realloc(M->items,M->s_items*sizeof(M->items[0])); 
    memset(M->items+M->n_items, 0, 
           (M->s_items-M->n_items)*sizeof(M->items[0]));
  }
  it= M->items + M->n_items;
  M->n_items++;
  return it;
}

#line 145 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void render_item
  (menu_item_t *it, char *text,
   int icon_width, int icon_height, uint32_t *icon_data)
{
  uint8_t *text_image; uint32_t text_width,text_height;
  int W,H; uint32_t *data;
  int i;
  
  text_render(menu_font, text, &text_width, &text_image);
  text_height= menu_font->height;

  H= text_height > MO_icon_h ? text_height : MO_icon_h;
  H= H > bi_menu_sub_height ? H : bi_menu_sub_height;
  W= MO_icon_w + MO_mid + text_width;

  data= malloc(sizeof(uint32_t)*W*H);
  for(i=0;i<W*H;i++) data[i]= MO_bg;

  draw_item_icon(W,H, data, icon_width, icon_height, icon_data);
  draw_item_text(W,H, data, text_width, menu_font->height, text_image);
  free(text_image);
  if (it->image) free(it->image);
  it->image= data;
  it->width= W;
  it->height= H;
}
#line 180 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void draw_item_icon(int W,int H, uint32_t *D,
                           int IW, int IH, uint32_t *ID)
{
  int ipitch;  // icon pitch in 4-byte words
  int IX, IY;  // source X and Y offset
  int DX, DY;  // destination X and Y offset

  ipitch= IW;

  IX= IY= 0;
  if (IW>MO_icon_w)
  {
    IX= (IW-MO_icon_w)/2;
    IW= MO_icon_w;
  }

  if (IH>MO_icon_h)
  {
    IY= (IH-MO_icon_h)/2;
    IH= MO_icon_h;
  }

  DX= (MO_icon_w-IW)/2;
  DY= (H-IH)/2;

  blend_alpha_image( D, W, H, DX, DY,
                     ID, ipitch, IH, IX, IY,
                       IW, IH, MO_bg);
}
#line 215 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void draw_item_text
      (int W,int H, uint32_t *D, int tW, int tH, uint8_t *tD)
{
  int DX, DY; // destination X and Y offset

  DX= MO_icon_w + MO_mid;
  DY= (H-tH)/2;

  graymap_draw( D,  W,  H, DX, DY,
               tD, tW, tH,  0, 0,
               MO_fg, MO_bg, tW, tH);
}

#line 394 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void create_menu_window(menu_t *M, int x,int y, int istop)
{
  XSetWindowAttributes attribs;
  unsigned int events;

  attribs.background_pixmap= None;
  attribs.override_redirect= True;
  M->xwindow= XCreateWindow( x_display, x_root_window,
                            x, y, M->width, M->height,
                            1, x_depth, InputOutput, 
                             x_visual, 
                           CWBackPixmap | CWOverrideRedirect, &attribs);

  events= ExposureMask | PointerMotionMask;
  if (istop) events |= StructureNotifyMask;
  XSelectInput(x_display, M->xwindow, events);

  if (!M->ximage)
    M->ximage= XCreateImage(x_display, x_visual, x_depth, ZPixmap, 0, 
                          (char*) M->image, M->width, M->height,
                                  32, M->width*4);
  M->x= x;
  M->y= y;

  Atom v[20];
  int i;

  i= 0;
  v[i++]= a_net_wm_window_type_popup_menu;
  set_window_property_atoms(M->xwindow, a_net_wm_window_type, i,v);
  XMapWindow(x_display, M->xwindow);

  int dummy_i, px, py; unsigned int dummy_u; Window dummy_w;

  XQueryPointer(x_display, M->xwindow, &dummy_w, &dummy_w, &dummy_i,
                &dummy_i, &px, &py, &dummy_u);

  if (px>=0 && px<M->width && py>=0 && py<M->height)
     menu_hilight_item(M, px,py);

  register_event_handler(M->xwindow, Expose, M, menu_expose_event,NULL);
  register_event_handler(M->xwindow, MotionNotify, M, menu_motion_event,NULL);
}

#line 440 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void menu_expose_event(menu_t *M, XExposeEvent *ev_expo)
{
  XPutImage(x_display,M->xwindow, DefaultGC(x_display, x_screen_number),
             M->ximage, ev_expo->x, ev_expo->y,
                       ev_expo->x, ev_expo->y,
            ev_expo->width, ev_expo->height);
}

static void menu_motion_event(menu_t *M, XMotionEvent *ev_moti)
{
  menu_hilight_item(M, ev_moti->x, ev_moti->y);
}

#line 458 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void menu_hilight_item(menu_t *M, int x,int y)
{
  int i;
  menu_item_t *it, *ai;
  
  for(i=0;i<M->n_items;i++)
    if (y>=M->items[i].y && y<M->items[i].y+M->items[i].height)
      break;
  if (i==M->n_items || i==M->active_item) return ;
  
  if (M->active_item!=-1)
  {
    ai= M->items+M->active_item;
    if (ai->type!=2) menu_hilight_bg(M,ai,1);
    if (ai->type==1 && ai->sub) undisplay_sub_menu(ai->sub);
  }
  it= M->items+i;
  if (it->type!=2) menu_hilight_bg(M,it,1);
  if (it->type==1 && it->sub) 
     display_sub_menu(it->sub, M->x + M->width, M->y+it->y);
  M->active_item= i;
}


static void menu_hilight_bg(menu_t *M,menu_item_t *it,int events)
{
  int r,c;
  for(r=0;r<it->height;r++)
     for(c=0;c<M->width;c++)
        M->image[ (it->y+r)*M->width+ c] ^= MO_hilight; 
  if (events)
    XClearArea(x_display,M->xwindow, 0, it->y, M->width, it->height, True);
}

#line 497 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void display_sub_menu(menu_t *M, int x,int y)
{
  if (y+M->height>=x_root_height)
     y= x_root_height - M->height;
  if (y<0) y= 0;
  create_menu_window(M,x,y,0);
}

static void undisplay_sub_menu(menu_t *M)
{
  if (!M) return ;
  if (M->active_item!=-1 && M->items[M->active_item].type==1)
     undisplay_sub_menu(M->items[M->active_item].sub);
  if (M->active_item!=-1)
     menu_hilight_bg(M, M->items+M->active_item,0);
  if (M->xwindow!=None)
  {
    XDestroyWindow(x_display,M->xwindow);
    unregister_event_handler(M->xwindow, Expose, NULL);
    unregister_event_handler(M->xwindow, MotionNotify, NULL);
  }
  M->xwindow= None;
  M->active_item= -1;
  if (!M->parent) 
  {
    unregister_event_handler(x_root_window, ButtonPress,
                               M->root_click_handler);
    //unregister_event_handler(x_root_window, ButtonRelease,
                               //M->root_release_handler);
  }
  M->root_click_handler= NULL;
  M->root_release_handler= NULL;
}

#line 546 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

static void menu_button_press(menu_t *root_menu, XButtonEvent *event)
{
  menu_t *M;
  int x,y;
  int i;
  x= event->x; y= event->y;
  M= root_menu;
  root_menu->selected= -1;

  while(1)
  {
    if (x>=M->x && x<M->x+M->width && y>=M->y && y<M->y+M->height)
       break;
    if (M->active_item==-1) goto notfound;
    if (M->items[M->active_item].type!=1) goto notfound;
    M= M->items[M->active_item].sub;
    if (!M) goto notfound;
  }

  y-= M->y;

  for(i=0;i<M->n_items;i++)
    if (y>=M->items[i].y && y<M->items[i].y+M->items[i].height)
       break;
  if (i==M->n_items || M->items[i].type!=0) return ;

  root_menu->selected= M->items[i].action;
notfound:
  XUngrabButton(x_display, Button1, AnyModifier, x_root_window);
  XAllowEvents(x_display, AsyncBoth, CurrentTime);
  syslog(LOG_USER | LOG_NOTICE, "UnGrabbing button1\n");
  XSync(x_display, False);
  undisplay_sub_menu(root_menu);
  return ;
}

#line 58 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

menu_t *menu_start()
{
  menu_t *M;
  M= calloc(1,sizeof(*M));
  M->s_items= 20;
  M->items= calloc(M->s_items,sizeof(M->items[0]));
  M->active_item= -1;
  M->xwindow= None;
  M->selected= -1;
  M->released= 0;
  return M;
}

#line 73 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

void menu_add_item(menu_t *M, int icon_width, int icon_height,
                              uint32_t *icon, char *text, int action)
{
  menu_item_t *it;
  it= alloc_item(M);
  it->type= 0;
  it->action= action;
  render_item(it, text, icon_width, icon_height, icon);
}

void menu_add_sub(menu_t *M, int icon_width, int icon_height,
                             uint32_t *icon, char *text, menu_t *sub)
{
  menu_item_t *it;
  it= alloc_item(M);
  it->type= 1;
  it->sub= sub;
  render_item(it, text, icon_width, icon_height, icon);
  if (sub) sub->parent= M;
}

void menu_add_sep(menu_t *M)
{
  menu_item_t *it;
  it= alloc_item(M);
  it->type= 2;
}

#line 240 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

void menu_end(menu_t *M)
{
  int W, H,i;
  int Y,r,c;
  menu_item_t *it;
  uint32_t *data;

  W= 0;
  H= MO_sepv;
  for(i=0;i<M->n_items;i++)
    if (M->items[i].type==2)
      H+= 2*MO_sepv;
    else
    {
      H+= M->items[i].height + MO_vspc;
      if (M->items[i].width>W) W= M->items[i].width;
    }

  W+= MO_left + MO_mid2 + bi_menu_sub_width + MO_right;
  data= malloc( W*H*sizeof(uint32_t) );

  for(i=0;i<W*H;i++) data[i]= MO_bg;

  Y= MO_sepv;
  for(i=0;i<M->n_items;i++)
  {
    it= M->items+i;
    it->y= Y;
    if (it->type==2)
    {
      Y+= MO_sepv;                       // FIXME: convert to server color
      for(c=MO_seph;c+MO_seph<W;c++) data[Y*W+ c] = MO_sepcolor;
      Y+= MO_sepv;
      continue;
    }
    if (!it->image) continue;
    for(r=0;r<it->height;r++)
      for(c=0;c<it->width;c++)
        data[ (Y+r)*W + (MO_left+c) ]= it->image[r*it->width+c];
    free(it->image);
    it->image= NULL;

    if (it->type==1)
      graymap_draw(data, W,H, W-bi_menu_sub_width- MO_right,
                         Y+(it->height-bi_menu_sub_height)/2, 
                   (uint8_t*) bi_menu_sub,
                   bi_menu_sub_width, bi_menu_sub_height, 0,0,
                   MO_fg, MO_bg,
                   bi_menu_sub_width, bi_menu_sub_height);
    Y+= it->height + MO_vspc;
  }

  if (M->image) free(M->image);
  M->image= data;
  M->width= W;
  M->height= H;
  M->released= 1;
}

#line 343 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

void menu_init(cfg_menu_style_t *cfg)
{
#define L(X)  if (cfg->G_##X) MO_##X= cfg->X;
  L(left) L(right) L(mid) L(mid2) L(vspc) L(icon_w) L(icon_h)
  L(font_size) L(bg) L(fg) L(seph) L(sepv) L(sepcolor) L(hilight)
  L(font_name)
#undef L
  menu_font= font_new(MO_font_name,MO_font_size);
}

#line 367 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

void menu_display_top(menu_t *M, int bot, int left,
                      Window *Rwindow, unsigned int *Revent)
{
  int x,y;
  x= left;
  y= x_root_height - bot - M->height;
  create_menu_window(M, x,y,1);

  XGrabButton(x_display, Button1, AnyModifier, x_root_window, False,
                 ButtonPressMask, // | ButtonReleaseMask,
               GrabModeAsync, GrabModeAsync, None, None);
  syslog(LOG_USER | LOG_NOTICE, "Grabbing button1\n");
  XSync(x_display, False);

  register_event_handler(x_root_window, ButtonPress, M, 
                      menu_button_press, &M->root_click_handler);
  //register_event_handler(x_root_window, ButtonRelease, M, 
                      //menu_button_release, &M->root_release_handler);

  M->selected= -1;
  *Rwindow= M->xwindow;
  *Revent= DestroyNotify;
  M->released= 0;
}

#line 589 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

int menu_get_selected(menu_t *M)
{
  return M->selected;
}

void menu_free(menu_t *M)
{
  int i;
  if (!M) return ;
  for(i=0;i<M->n_items;i++)
    switch(M->items[i].type)
    {
    case 1: 
       menu_free(M->items[i].sub);
    case 0: 
       if (M->items[i].image) free(M->items[i].image);
       break; 
    }
  free(M->items);
  if (M->ximage) XDestroyImage(M->ximage);
  else if (M->image) free(M->image);
  if (M->root_click_handler)
     unregister_event_handler(x_root_window, ButtonPress,
                                 M->root_click_handler);
  //if (M->root_release_handler)
     //unregister_event_handler(x_root_window, ButtonRelease,
                                 //M->root_release_handler);
  free(M);
}

#line 625 "/home/dodo/tmp/mupanel-orig-1.5/src/menu.u"

int menu_is_released(menu_t *M)
{
  return M->released;
}

void menu_release(menu_t *M)
{
  M->released= 1;
}

