#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include "x-basics.h"
#include "x-events.h"
#include "x-image.h"
#include "fsel.h"
#include "graymap.h"
#include "chdir.h"
#include "btn.h"

#include "chdir-scr.c"

static int fc_butpos[]=
{ 45, 36, 176, 34 , 'O',
  10, 449, 65, 24 , 'C' };



typedef struct
{
  Window window; int w,h;
  ximage_t *image;
  int selbut[5];
  void (*func)(void *, char *);
  void *arg;
  file_selector_t *fs;
} fdir_t;

static void fc_destroy(fdir_t *fc, XEvent *ev);

static void fc_press(fdir_t *fc, XButtonEvent *ev)
{
  btn_click(fc->window, fc->image, fc_butpos, 10, ev, fc->selbut);
}

static void fc_release(fdir_t *fc, XButtonEvent *ev)
{
  Window window;
  switch(btn_release(fc->window, fc->image, ev, fc->selbut))
  {
  case 'O':
     if (fc->func) (*fc->func)(fc->arg, file_selector_directory(fc->fs));
  case 'C':
     window= fc->window;
     file_selector_free(fc->fs);
     fc_destroy(fc,NULL);
     XDestroyWindow(x_display, window);
     break;
  }
}

static void fc_destroy(fdir_t *fc, XEvent *ev)
{
  destroy_image(fc->image);
  unregister_event_handler(fc->window, Expose, NULL);
  unregister_event_handler(fc->window, ButtonPress, NULL);
  unregister_event_handler(fc->window, ButtonRelease, NULL);
  unregister_event_handler(fc->window, DestroyNotify, NULL);
  free(fc);
  printf("dir selector destroyed\n");
}


static void fc_expose(fdir_t *fc, XExposeEvent *ev)
{
  put_image(fc->window, ev->x, ev->y, fc->image,ev->x, ev->y, 
            ev->width, ev->height);
}

static void click_file(void *arg, file_selector_t *fs, char *sel)
{
  // IGNORED
}

void choose_dir(Window parent,
                 char *dir, void (*finish)(void *, char *), void *arg)
{
  XSetWindowAttributes attribs;
  fdir_t *fc;
  int w,h;

  w= bi_fdir_width;
  h= bi_fdir_height;

  fc= calloc(1,sizeof(*fc));
  fc->image= create_image(w,h);
  fc->w= w;
  fc->h= h;

  fc->func= finish;
  fc->arg= arg;

  attribs.background_pixmap= None;
  fc->window= XCreateWindow(x_display, parent, 0,0,w,h,
                            0, x_depth, InputOutput, x_visual,
                            CWBackPixmap, &attribs);
  
  XSelectInput(x_display, fc->window, ExposureMask | 
                       ButtonPressMask | ButtonReleaseMask | 
                        StructureNotifyMask);
  register_event_handler(fc->window, Expose, fc, fc_expose, NULL);
  register_event_handler(fc->window, ButtonPress, fc, fc_press, NULL);
  register_event_handler(fc->window, ButtonRelease, fc, fc_release, NULL);
  register_event_handler(fc->window, DestroyNotify, fc, fc_destroy, NULL);

  fill_rect(fc->image->data, fc->w, fc->h, 0,0, fc->w, fc->h, 0xffffff);

  blend_alpha_image(fc->image->data, fc->w, fc->h,  0,0,
       (uint32_t*)  bi_fdir, bi_fdir_width, bi_fdir_height,
                    0,0, bi_fdir_width, bi_fdir_height, 0xffffff);
  
  fc->fs= file_selector_new(fc->window, 8,78, 621, 364,
                            dir, 0, NULL, click_file, fc);
  XMapWindow(x_display, fc->window);
}

