#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include "x-basics.h"
#include "x-events.h"
#include "x-image.h"
#include "fsel.h"
#include "graymap.h"
#include "chicon.h"
#include "btn.h"
#include "imgload.h"

#include "chicon-scr.c"

typedef struct
{
  Window window; int w,h;
  ximage_t *image;
  char *selected;
  int butpos[10];
  int selbut[5];
  void (*func)(void *, char *);
  void *arg;
  file_selector_t *fs;
  int icox, icoy;
} ich_t;

static void fc_destroy(ich_t *fc, XEvent *ev)
{
  if (fc->selected) free(fc->selected);
  destroy_image(fc->image);
  unregister_event_handler(fc->window, Expose, NULL);
  unregister_event_handler(fc->window, ButtonPress, NULL);
  unregister_event_handler(fc->window, ButtonRelease, NULL);
  unregister_event_handler(fc->window, DestroyNotify, NULL);
  free(fc);
}

static void fc_press(ich_t *fc, XButtonEvent *ev)
{
  btn_click(fc->window, fc->image, fc->butpos, 10, ev, fc->selbut);
}

static void fc_release(ich_t *fc, XButtonEvent *ev)
{
  Window window;
  switch(btn_release(fc->window, fc->image, ev, fc->selbut))
  {
  case 1: if (!fc->selected) return ;
          if (fc->func) (*fc->func)(fc->arg, fc->selected);
  case 2: window= fc->window;
          file_selector_free(fc->fs);
          fc_destroy(fc, NULL);
          XDestroyWindow(x_display, window);
          break;
  }
}


static void fc_expose(ich_t *fc, XExposeEvent *ev)
{
  put_image(fc->window, ev->x, ev->y, fc->image,ev->x, ev->y, 
            ev->width, ev->height);
}
#define ICOS 32

static void click_file(void *arg, file_selector_t *fs, char *sel)
{
  ich_t *fc;
  unsigned int icow,icoh, *icod;
  int dx,dy, sx, sy, cw, ch;

  fc= arg;
  if (fc->selected) free(fc->selected);
  fc->selected= strdup(sel);

  fill_rect(fc->image->data, fc->w, fc->h, fc->icox, fc->icoy,
                                     ICOS, ICOS, 0xffffff);

  XClearArea(x_display,fc->window, fc->icox, fc->icoy, ICOS, ICOS, True);

  if (load_icon(sel, ICOS, ICOS, &icow, &icoh, &icod)) return ;

  dx= fc->icox; dy= fc->icoy;
  sx= 0;        sy= 0;
  cw= icow;     ch= icoh; 
  if (icow<ICOS)
  {
    dx+= (ICOS-icow)/2;
  }
  else if (icow>ICOS)
  {
    sx= (icow-ICOS)/2;
    cw= ICOS;
  }

  if (icoh<ICOS)
  {
    dy+= (ICOS-icoh)/2;
  }
  else if (icoh>ICOS)
  {
    sy= (icoh-ICOS)/2;
    ch= ICOS;
  }

  blend_alpha_image(fc->image->data, fc->w, fc->h,  dx, dy,
                    icod, icow, icoh, sx, sy,
                    cw, ch, 0xffffff);
  free(icod);
}

void choose_icon(Window parent, int x,int y, int w, int h,
                 char *dir, void (*finish)(void *, char *), void *arg)
{
  XSetWindowAttributes attribs;
  ich_t *fc;
  int i,dx,dy;
  static char *sfx[]= { ".png", NULL };

  fc= calloc(1,sizeof(*fc));
  fc->image= create_image(w,h);
  fc->w= w;
  fc->h= h;

  fc->func= finish;
  fc->arg= arg;

  attribs.background_pixmap= None;
  fc->window= XCreateWindow(x_display, parent, x, y, w, h,
                            0, x_depth, InputOutput, x_visual,
                            CWBackPixmap, &attribs);
  
  XSelectInput(x_display, fc->window, ExposureMask | KeyPressMask |
                       StructureNotifyMask |
                       ButtonPressMask | ButtonReleaseMask);
  register_event_handler(fc->window, Expose, fc, fc_expose, NULL);
  register_event_handler(fc->window, ButtonPress, fc, fc_press, NULL);
  register_event_handler(fc->window, ButtonRelease, fc, fc_release, NULL);
  register_event_handler(fc->window, DestroyNotify, fc, fc_destroy, NULL);

  fill_rect(fc->image->data, fc->w, fc->h, 0,0, fc->w, fc->h, 0xffffff);

  dx= (w - bi_fico_width)/2;
  dy= (h - bi_fico_height);

  blend_alpha_image(fc->image->data, fc->w, fc->h,   dx, dy,
       (uint32_t*)  bi_fico, bi_fico_width, bi_fico_height,
                    0,0, bi_fico_width, bi_fico_height, 0xffffff);
  
  fc->fs= file_selector_new(fc->window, 0,0, w, dy,
                            dir, 0, sfx,
                           click_file, fc);

  i= 0;
  fc->butpos[i++]= dx + 7;
  fc->butpos[i++]= dy + 8;
  fc->butpos[i++]= 64;
  fc->butpos[i++]= 26;
  fc->butpos[i++]= 1; // OK

  fc->butpos[i++]= dx + 127;
  fc->butpos[i++]= dy + 8;
  fc->butpos[i++]= 64;
  fc->butpos[i++]= 26;
  fc->butpos[i++]= 2;  // Cancel

  fc->icox= dx + 84;
  fc->icoy= dy + 6;

  XMapWindow(x_display, fc->window);
}

