#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <ctype.h>

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include "x-basics.h"
#include "x-events.h"
#include "x-image.h"
#include "txtren.h"
#include "graymap.h"
#include "ptrbuf.h"

#include "fsel.h"
#include "fontsel.h"

typedef struct { int x,y,w,h;  } rect_t;

typedef struct { int x1, y1, x2, y2; } posrect_t;

static posrect_t  bt_cancel= { 6, 6, 84, 43 };
static posrect_t  bt_ok=     { 6, 437, 84, 474 };
static posrect_t  filarea=   { 6, 48, 634, 432 };
static posrect_t  samarea=   { 91, 437, 634, 474 };

#define FOW  640
#define FOH  480

struct xfontsel
{
  Window window;
  ximage_t *image;
  file_selector_t* fs;
  void (*click)(void *,font_selector_t*,int);
  void *arg;
};

static void fos_expose(font_selector_t *fs, XExposeEvent *ev);
static void fos_press(font_selector_t *fs, XButtonEvent *ev);

static void preview_font(void *arg, file_selector_t *fs, char *path);

static char *font_suffixes[]=
{
  ".ttf", ".otf", NULL
};

#include "fontsel-scr.c"

font_selector_t *font_selector_new
     (Window parent, int x,int y,
      void (*click)(void *,font_selector_t*,int), void *arg)
{
  font_selector_t *fos;
  XSetWindowAttributes attribs;

  fos= calloc(1,sizeof(*fos));
  fos->click= click; fos->arg= arg;

  attribs.background_pixmap= None;
  fos->window= XCreateWindow(x_display, parent, x,y, FOW,FOH,
                            0, x_depth, InputOutput, x_visual,
                            CWBackPixmap, &attribs);
  fos->image= create_image(FOW, FOH);
  XSelectInput(x_display, fos->window, ExposureMask | ButtonPressMask);
  register_event_handler(fos->window, Expose, fos, fos_expose, NULL);
  register_event_handler(fos->window, ButtonPress, fos, fos_press, NULL);

  blend_alpha_image(fos->image->data, FOW, FOH, 0,0,
        (uint32_t*) bi_foscreen, FOW, FOH, 0,0,
               0xffffff, FOW, FOH);

  fos->fs= file_selector_new(fos->window,
               filarea.x1, filarea.y1, 
               filarea.x2-filarea.x1+1, filarea.y2-filarea.y1+1, 
               "/usr/share/fonts", 0, font_suffixes, preview_font, fos);
  XMapWindow(x_display, fos->window);
  return fos;
}

static void fos_expose(font_selector_t *fos, XExposeEvent *ev)
{
  put_image(fos->window, ev->x, ev->y,
            fos->image, ev->x, ev->y,
            ev->width, ev->height);
}


static int in_prect(XButtonEvent *ev, posrect_t *R)
{
  if (ev->x<R->x1) return 0;
  if (ev->y<R->y1) return 0; 
  if (ev->x>R->x2) return 0;
  if (ev->y>R->y2) return 0;
  return 1;
}

static void fos_press(font_selector_t *fs, XButtonEvent *ev)
{
  int V;
  if (ev->button!=Button1) return ;
  if (in_prect(ev, &bt_ok))  V= 1;
  else if (in_prect(ev, &bt_cancel)) V= 0;
  else return ;
  if (fs->click)
    (*fs->click)(fs->arg, fs, V);
}

static void preview_font(void *arg, file_selector_t *fs, char *path)
{
   u_font_t *font;
   uint8_t *bmap;
   uint32_t W;
   font_selector_t *fos;

   int Xd, Yd, Xs, Ys;
   int Wd, Hd, Ws, Hs;
   int Wr, Hr;

   fos= arg;
   font= font_new(path, 12);

   Wd= samarea.x2-samarea.x1-1;
   Hd= samarea.y2-samarea.y1-1;

   fill_rect(fos->image->data, FOW, FOH, samarea.x1+1, samarea.y1+1, 
                             Wd-1, Hd-1, 0xffffff);

   if (!font) goto done;

   text_render(font, "The quick brown fox jumped over the lazy dog.", &W, &bmap);

   Hs= font->height;
   Ws= W; 

   font_free(font);

   if (Ws>Wd) { Wr= Wd; Xd= 0; Xs= (Ws-Wd)/2; }
         else { Wr= Ws; Xd= (Wd-Ws)/2; Xs= 0; }

   if (Hs>Hd) { Hr= Hd; Yd= 0; Ys= (Hs-Hd)/2; }
         else { Hr= Hs; Yd= (Hd-Hs)/2; Ys= 0; }

   graymap_draw(fos->image->data, FOW, FOH, Xd+samarea.x1, Yd+samarea.y1, 
                bmap, Ws, Hs, Xs, Ys,
                0x0, 0xffffff, Wr, Hr);

   free(bmap);
done:
   XClearArea(x_display, fos->window, samarea.x1+1, samarea.y1+1,
                                              Wd+1, Hd+1, True);
}


void font_selector_free(font_selector_t *fos)
{
  file_selector_free(fos->fs);
  unregister_event_handler(fos->window, Expose, NULL);
  unregister_event_handler(fos->window, ButtonPress, NULL);
  XDestroyWindow(x_display, fos->window); 
  destroy_image(fos->image);
  free(fos);
}

