#include <stdlib.h>
#include <string.h>
#include "ptrbuf.h"


struct pbufnode { 
  struct pbufnode *next;
  size_t len;
  void* data[];
};

struct ptrbuf {
  struct pbufnode *first, *last;
  size_t blksize;
};


_PTRBUF_PROTO ptrbuf_t *ptrbuf_new(size_t blksize)
{
  ptrbuf_t *B;
  B= malloc(sizeof(*B));
  B->first= B->last= NULL;
  B->blksize= blksize;
  return B;
}


_PTRBUF_PROTO void ptrbuf_free(ptrbuf_t *B, void ***Rd, size_t *Rl)
{
  struct pbufnode *P,*N;
  if (Rd || Rl) ptrbuf_collect(B, Rd, Rl);
  for(P=B->first;P;P=N)
  {
    N= P->next;
    free(P);
  }
  free(B);
}

_PTRBUF_PROTO void ptrbuf_collect(ptrbuf_t *B, void ***Rd, size_t *Rl)
{
  size_t S;
  struct pbufnode *P;
  void **R;

  if (!Rd && !Rl) return ;

  S= 0;
  for(P=B->first;P;P=P->next) S+= P->len;

  if (S==0) { if (Rd) *Rd= NULL; if (Rl) *Rl= 0; return ; }
  if (!Rd) { *Rl= S; return ; }

  R= malloc(S*sizeof(void*));
  S= 0;
  for(P=B->first;P;P=P->next)
  {
    memcpy(R+S, P->data, P->len*sizeof(void*));
    S+= P->len;
  }

  *Rd= R;
  if (Rl) *Rl= S;
}

_PTRBUF_PROTO void ptrbuf_put(ptrbuf_t *B, void *d)
{
  struct pbufnode *P;

  P= B->last;
  if (!P || P->len==B->blksize)
  {
      P= malloc(sizeof(*P)+B->blksize*sizeof(void*));
      P->len= 0;
      P->next= NULL;
      if (!B->last) { B->last= B->first= P; }
      else { B->last->next= P; B->last= P; }
 }

  P->data[P->len]= d;
  P->len++;
}

