#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "qumap.c"


void dump_map(qumap_t *map);


void dump_node(qumap_node_t *node)
{
  if (node->internal) {
    printf("[%d,", node->bit);
    dump_node(Q_CHLD(node,0));
    printf(",");
    dump_node(Q_CHLD(node,1));
    printf("]");
 } else {
    char g[10];
    uint64_t V;
    int i;
    V= Q_KEY(node);
    for(i=0;i<8;i++) { g[7-i]= V & 0xff; V>>=8; } 
    g[8]= 0;
    printf("(%s)", g);
 }
    
}

void dump_map(qumap_t *map)
{
  if (map->root) dump_node(map->root);
  else printf("[EMPTY]");
}

#define JJ 20

typedef struct {
  char str[JJ+1];
  int value;
} aa_t;

#define N 10000
#define M 20000

qumap_t *s;
aa_t bb[N];
aa_t *in[N];
aa_t *out[N];
int n_in;
int n_out;

int n_insert,n_delete,n_lookup;

uint64_t quval(char *k)
{
  unsigned char *Z;
  uint64_t S;
  int i;
  Z= (unsigned char*) k;
  S= 0;
  for(i=0;i<8;i++) S= (S<<8) | Z[i];
  return S;
}

void fail()
{
  printf("%d insertions, %d lookups, %d deletions\n",
       n_insert, n_lookup, n_delete);
  dump_map(s);
  printf("\n");
  fflush(0);
  *(int*)0= 0;
}

void prepare(int k)
{
  int i;
  int L;
  bb[k].value= k;
again:
  L= 8;
  for(i=0;i<L;i++)
     bb[k].str[i]= rand()%26+'a';
  bb[k].str[i]= 0;

  for(i=0;i<k;i++) if (!strcmp(bb[k].str,bb[i].str)) goto again;
  out[k]= bb+k;
  n_out++;
}


void do_find()
{
  if (!n_in) return ;
  n_lookup++;
  int K= rand()%n_in;

  printf("L> %s\n", in[K]->str);

  aa_t *R= qumap_find(s,quval(in[K]->str));
  if (R!=in[K])
    { printf("string %s in table but not found. \n"
             "   returned ptr= %p\n", in[K]->str, R ); fail(); }

  dump_map(s); printf("\n");
}


void do_remove()
{
  if (!n_in) return ;
  n_delete++;
  int K= rand()%n_in;

  printf("R> %s\n", in[K]->str);

  aa_t *R= qumap_remove(s,quval(in[K]->str));
  if (R!=in[K]) 
    { printf("string %s in table but not found.\n", in[K]->str); fail(); }


  aa_t *Z;
  Z= in[K];
  in[K]= in[n_in-1];
  out[n_out]= Z;
  n_in--; n_out++;

  R= qumap_find(s, quval(Z->str));
  if (R)
  {
    printf("string %s was removed, but still can be found.\n",
           Z->str); fail(); 
  }
  dump_map(s); printf("\n");
}

void do_insert()
{
  if (!n_out) return ;
  n_insert++;
  int K= rand()%n_out;

  printf("I> %s\n", out[K]->str);

  aa_t *R= qumap_insert(s,quval(out[K]->str), out[K]);
  if (R)
    { printf("tried to insert string %s but it's already there.\n",
      out[K]->str);
      printf("returned pointer= %p (%s,%d)\n"
              "tried pointer=    %p (%s,%d)\n",
           R, R->str, R->value,
           out[K], out[K]->str, out[K]->value);
      fail(); }


  aa_t *Z;
  Z= out[K];
  out[K]= out[n_out-1];
  in[n_in]= Z;
  n_in++; n_out--;


  R= qumap_find(s, quval(Z->str));
  if (!R)
  {
     printf("string %s was just inserted, but it can't be found.\n",
        Z->str);
     fail();
  }
  dump_map(s); printf("\n");
}

int main()
{
  int i,Y;
  FILE *plot;
  plot= fopen("plot.txt", "w");
  s= qumap_new();
  srand(time(0));
  for(i=0;i<N;i++)
    prepare(i);
  Y= 0;
  // do_insert(); do_insert(); do_remove(); do_find(); return 0;
  for(i=0;i<M;i++)
  {
    int op= rand()%100;
    if (op<30) do_find();
    else if (op<60) do_insert();
    else do_remove();
    fprintf(plot,"%d %d\n", Y, n_in);
    Y++;
  }
  fclose(plot);
  aa_t *Z;
  while((Z=qumap_destroy(s))) {
    printf("<D %s\n", Z->str);
    dump_map(s);
    printf("\n");
  }
  free(s);
  printf("%d insertions, %d lookups, %d deletions\n",
       n_insert, n_lookup, n_delete);
  return 0;
}



