#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <string.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>

#include "x-basics.h"
#include "x-props.h"
#include "x-image.h"
#include "x-events.h"
#include "config.h"
#include "menu.h"
#include "imgload.h"
#include "txtren.h"
#include "appmenu.h"
#include "panel.h"
#include "p-start.h"
#include "p-box.h"
#include "pager.h"
#include "taskbar.h"
#include "clock.h"
#include "graymap.h"


/****************************************************************************
  The panel is a simple window containing other windows. We record the
  windows along with their order & properties. When resizing some of
  the stuff, I will need to reposition the other windows.

  I simply put them in order and then expand the taskbar window as much as
  it goes. The anchoring to the left or right will result automatically.

****************************************************************************/

static panel_part_t parts[20];
static int n_parts;

static Window panel_window;
static uint32_t PO_bg           = 0xededed;
static uint32_t PO_height       = 32;
static uint32_t PO_fg           = 0;
static uint32_t PO_spacing      = 2;
static uint32_t PO_button_size  = 24;
static uint32_t PO_right        = 0;
static uint32_t PO_left         = 0;
static uint32_t PO_clock_on     = 1;
static uint32_t PO_override_redirect= 0;

static int panel_y, panel_w, panel_h;


static void position_panel_windows()
{
  int i,x;
  x= 0;
  for(i=0;i<n_parts;i++)
  {
    if (parts[i].window==None) continue;
    parts[i].x= x;
    XMoveWindow(x_display, parts[i].window, parts[i].x, 0);
    XSync(x_display, False);
    x+= parts[i].w + PO_spacing;
  }
}

static void set_panel_window_properties()
{
  Atom v[20];
  int i;
  unsigned long strut[12];
  XSizeHints *hints;

  i= 0;
  v[i++]= a_net_wm_window_type_dock;
  // v[i++]= a_net_wm_window_type_normal;
  set_window_property_atoms(panel_window, a_net_wm_window_type, i,v);

  i= 0;
  v[i++]= a_net_wm_state_sticky;
  v[i++]= a_net_wm_state_skip_taskbar;
  v[i++]= a_net_wm_state_skip_pager;
  v[i++]= a_net_wm_state_above;
  set_window_property_atoms(panel_window, a_net_wm_state, i, v);


  hints= XAllocSizeHints();
  hints->flags= PMinSize | PMaxSize | PWinGravity | USPosition | USSize;
  hints->min_width= hints->max_width= panel_w;
  hints->min_height= hints->max_height= panel_h;
  hints->win_gravity= StaticGravity;

  hints->x= PO_left;
  hints->y= panel_y;
  hints->width= panel_w;
  hints->height= panel_h;
  XSetWMNormalHints(x_display, panel_window, hints);
  XSync(x_display, False);
  XFree(hints);

  /**************************************************
    From the netwm spec, the order of the fields is:
       left, right, top, bottom,
       left_start_y, left_end_y,
       right_start_y, right_end_y,
       top_start_x, top_end_x,
       bottom_start_x, bottom_end_x
  **************************************************/
  memset(strut, 0, sizeof(strut));
  strut[3]= panel_h;
  strut[10]= PO_left;
  strut[11]= strut[10] + panel_w-1;
  set_window_property_cardinals(panel_window, a_net_wm_strut_partial,
                                12, strut);

  /**************************************************
    I will also set the old strut values to
    accomodate older window managers.
  **************************************************/
  set_window_property_cardinals(panel_window, a_net_wm_strut, 4, strut);
  set_window_property_string(panel_window, a_wm_window_role, "Panel");
}

static void compute_taskbar_width()
{
  int i, fw,j;
  fw= 0;
  j= 0;
  for(i=0;i<n_parts;i++) 
  {
     if (parts[i].window==None || i==2) continue;
     fw+= parts[i].w;
     if (j) fw+= PO_spacing;
     j++;
  }
  parts[2].w= panel_w - fw;
}

void init_panel(char *appmenu_fname, cfg_top_t *cfg)
{
  XSetWindowAttributes attribs;
  unsigned long        attribmask;

#define LL(X) if (cfg->G_##X) PO_##X= cfg->X;
  LL(clock_on) LL(height) LL(bg) LL(fg) LL(right) LL(left) 
  LL(spacing) LL(button_size) LL(override_redirect)
#undef LL

  panel_h= PO_height;
  panel_w= x_root_width - PO_left - PO_right;
  panel_y= x_root_height - panel_h; // - 100;

  attribs.background_pixel= server_color(PO_bg);
  attribs.override_redirect= True;

  attribmask= CWBackPixel;
  if (PO_override_redirect)
     attribmask |=  CWOverrideRedirect;
  panel_window= XCreateWindow( x_display, x_root_window,
                               PO_left, panel_y, panel_w, panel_h,
                               0, x_depth, InputOutput, x_visual,
                               attribmask, &attribs);
  XSelectInput(x_display, panel_window, ExposureMask | ButtonPressMask);

  set_panel_window_properties();

  init_startmenu (panel_window,panel_y,panel_h,
                  appmenu_fname, &parts[0].w, &parts[0].window,PO_fg, PO_bg);

  init_box(panel_window, panel_h, 0, cfg->left_box_file,
           PO_button_size, PO_spacing, PO_bg, parts+1);

  parts[2].type= 1; 

  init_box(panel_window, panel_h, 1, cfg->right_box_file,
           PO_button_size, PO_spacing, PO_bg, parts+3);

  init_pager(panel_window, panel_h, panel_y, &cfg->pager_style,
                &parts[4].window, &parts[4].w);

  if (PO_clock_on)
    init_clock(panel_window, panel_h, panel_y, &cfg->clock,
                           &parts[5].window,  &parts[5].w);

  n_parts= 6;

  compute_taskbar_width();
  init_taskbar(panel_window, PO_left, panel_y, parts[2].w, panel_h, 
                 &cfg->taskbar, &parts[2].window);

  position_panel_windows();
  XMapWindow(x_display, panel_window);
}


void pager_resized()
{
  compute_taskbar_width();
  taskbar_resize(parts[2].w, panel_h);
  position_panel_windows();
}

void panel_loop()
{
  int x,R;
  XEvent event;

  while(1) {
    while(!PO_clock_on)
    {
      XEvent event;
      XNextEvent(x_display,&event);
      handle_registered_event(&event);
    }

    struct timeval tv;
    tv.tv_sec= 60;
    tv.tv_usec= 1;
    x= ConnectionNumber(x_display);
    XSync(x_display,False);

    while(PO_clock_on)
    {
      while(XPending(x_display))
      {
        XNextEvent(x_display,&event);
        handle_registered_event(&event);
      }

      XFlush(x_display);
      fd_set rfd, xfd, wfd;

      if (tv.tv_sec==0) break;
      FD_ZERO(&rfd);  FD_ZERO(&xfd); FD_ZERO(&wfd);
      FD_SET(x, &rfd); FD_SET(x, &xfd);

      R= select( x+1, &rfd, &wfd, &xfd, &tv);
      if (R<0)
      {
        if (errno==EINTR) { usleep(1000); break; }
        return ; // FIXME: log this stuff.
      }
      if (FD_ISSET(x, &xfd)) return ; // FIXME: log this stuff.

      if (FD_ISSET(x, &rfd))
      {
        XNextEvent(x_display,&event);
        handle_registered_event(&event);
      }

      update_clock();
      handle_image_chores();
    }
  }
}
