
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "x-basics.h"
#include "x-props.h"
#include "x-misc-props.h"

static unsigned long n_desktops;

int get_number_of_desktops(uint32_t *D)
{
  unsigned long V;
  if (n_desktops) { *D= n_desktops; return 0; }
  if (get_window_property_cardinal(None, a_net_number_of_desktops, &V))
     return 1;
  n_desktops= *D= V;
  return 0;
}

int get_current_desktop(uint32_t *D)
{
  unsigned long V;
  if (get_window_property_cardinal(x_root_window, a_net_current_desktop, &V))
     return 1;
  else *D= V;
  return 0;
}

int set_current_desktop(int nr)
{
  XEvent event;

  event.xclient.type = ClientMessage;
  event.xclient.window = x_root_window;
  event.xclient.message_type= a_net_current_desktop;
  event.xclient.format = 32;
  event.xclient.data.l[0] = nr;
  event.xclient.data.l[1] = 0;
  event.xclient.data.l[2] = 0;
  event.xclient.data.l[3] = 0;
  event.xclient.data.l[4] = 0;

  if (XSendEvent(x_display, x_root_window, False,
                 SubstructureNotifyMask | SubstructureRedirectMask, &event))
    return 0;
  return 1;
}

void change_window_desktop(Window window, int desk)
{
  XEvent event;
  memset(&event, 0, sizeof(event));
  
  event.xclient.type = ClientMessage;
  event.xclient.window = window;
  event.xclient.message_type= a_net_wm_desktop;
  event.xclient.format = 32;
  event.xclient.data.l[0] = desk;
  event.xclient.data.l[1] = 2;

  XSendEvent(x_display, x_root_window, False,
             SubstructureNotifyMask | SubstructureRedirectMask, &event);
}

char *get_active_window_manager_name()
{
  Window child;
  Window child_ref;
  char *child_name;
  size_t dummy;
  if (get_window_property_window(None, a_net_supporting_wm_check, &child))
     return NULL;
  if (get_window_property_window(child, a_net_supporting_wm_check, &child_ref))
     return NULL;
  if (child_ref!=child) return NULL;
  if (get_window_property_utf8_string(child, a_net_wm_name,
                                       &dummy, &child_name))
     return NULL;
  return child_name;
}


void request_close_window(Window window)
{
  XEvent event;
  memset(&event, 0, sizeof(event));
  
  event.xclient.type = ClientMessage;
  event.xclient.window = window;
  event.xclient.message_type= a_wm_protocols;
  event.xclient.format = 32;
  event.xclient.data.l[0] = a_wm_delete_window;

  XSendEvent(x_display, window, False, 0 , &event);
}

int get_active_window(Window *Rwindow)
{
  size_t n;
  Window *windows;
  if (get_window_property_windows(x_root_window, a_net_active_window,
                                  &n, &windows, 1) || n==0)
    return 1;
  *Rwindow= windows[0];
  free(windows);
  return 0;
}

void set_active_window(Window window)
{
  XEvent event;

  XRaiseWindow(x_display, window);

  event.xclient.type = ClientMessage;
  event.xclient.window = window;
  event.xclient.message_type= a_net_active_window;
  event.xclient.format = 32;
  event.xclient.data.l[0] = 2;
  event.xclient.data.l[1] = 0;
  event.xclient.data.l[2] = 0;
  event.xclient.data.l[3] = 0;
  event.xclient.data.l[4] = 0;

  XSendEvent(x_display, x_root_window, False,
               SubstructureNotifyMask | SubstructureRedirectMask, &event);
}


void number_of_desktops_changed()
{
  unsigned long V;
  if (get_window_property_cardinal(None, a_net_number_of_desktops, &V))
     return ;
  n_desktops= V;
}
